/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Mon Apr 12 20:53:42 CEST 2004
    copyright            : (C) 2004 by Marco Savegnago IW3FQG
    email                : msave@tin.it

  SVClick is simple program (the my first one on linux...)
  that download camera photo from supervozelj node using
  ax25 stream and convert it to jpeg format.

  Part of the code was from:
  
  svphoto - Read SuperVozelj photo, and write it in a readable
    jpeg or png format
    Copyright (C) 2001  Paolo Subiaco

 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include  "svclick.h"
#include <unistd.h>

#include <sys/types.h>
#include <utime.h>
#include <time.h>
#include <ctype.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/time.h>

#include <math.h>


int     i_AX25Connection;
byte    by_Buffer[MAXREAD];

int     npar; // used to parse argv[]

struct  jpeg_compress_struct  cinfo;
struct  jpeg_error_mgr  jerr;

char  sz_Banner[]       = "SVClick Version %s\n";
char  scz_ThisVersion[] = "1.0";
char  sz_Usage[]       = "SVClick Usage: SVCLICK port node\n";

void term_handler(int i)
{
  printf("SIGTERM: Terminate svclick (i=%d)\n",i);
  close (i_AX25Connection);
  jpeg_destroy_compress(&cinfo);

  exit(i);
}

void  Usage(void)
{
  fprintf(stderr,sz_Usage);  
}


int main(int na, char **a)
{
  int narg;
  char *p_arg;
  int i;
  int j;
  char*   sz_Ax25Port   = NULL;
  char*   sz_SVNodeName = NULL;

  time_t t_watchDog;
  time_t t_startTime;  // local time in seconds
  struct tm *t_localTime;
  int i_diffTime;	    // difference between local and gmt time

  int nread;
  char sz_line[MAXREAD];
  byte by_imageMap[YMAX][XMAX];

  char  nodeprompt[12];
  int   xoffset;
  int   yoffset;
  int   xlen;
  int   ylen;
  int   x;
  int   y;
  int   nly;

  char  sz_OutFile[PATH_MAX];

  int i_quality = 100;

  npar=1;

  fprintf(stdout, sz_Banner, scz_ThisVersion);

  for(narg = 1; narg < na; narg++)
  {
    p_arg=a[narg];

    if(*p_arg == '/' || *p_arg == '-')
    {
      p_arg++;
      switch(toupper(*p_arg))
      {
        case  'Q':
          i_quality = atoi(++p_arg);
          break;
        case  '?':
        case  'H':
        default:
          Usage();
          return  -1;
          break;
      }
    }
    else if(sz_Ax25Port == NULL)
      sz_Ax25Port=p_arg;
    else if(sz_SVNodeName == NULL)
    {
      sz_SVNodeName=p_arg;

      for (i=0;i<strlen(sz_SVNodeName);i++)
        sz_SVNodeName[i]=toupper(sz_SVNodeName[i]);
    }
    else
    {
      Usage();
      return  -1;
    }

  }

  signal(SIGTERM,&term_handler);

  i_diffTime=0;
  t_startTime=time(NULL);
  t_localTime=localtime(&t_startTime);
  i_diffTime=timezone-3600*t_localTime->tm_isdst;

  t_startTime=time(NULL);
  t_localTime=localtime(&t_startTime);

  sprintf(by_Buffer,"\n%s (difftime=%d): SVClick program is starting....\n",ctime(&t_startTime),i_diffTime);

  fprintf(stderr,"\n=====================================%s",by_Buffer);

  t_startTime-=i_diffTime;	//now t_startTime will be the local time.

  //  Check if /etc/ax25/axports is properly configured

  if(AX25CheckConfig(sz_Ax25Port))
  {
    fprintf(stderr, "Error on AX25CheckConfig\n");
    return 1;
  }

  if ((i_AX25Connection=AX25ConnectTo(sz_Ax25Port, sz_SVNodeName))==-1)
  {
    fprintf(stdout, "Error connecting %s\n",sz_SVNodeName);
    return -1;
  }

  fcntl(i_AX25Connection,F_SETFL,O_NONBLOCK); //set the socket in non-blocking mode

  if (AX25WaitString(i_AX25Connection, sz_SVNodeName, sz_line, WAIT_TIMEOUT))
  {
    // ok, i'm connected to the argv[i] node
    // initialize the x coordinates
    xoffset=XOFFSET;
    yoffset=YOFFSET;

    xlen=XLEN;
    ylen=YLEN;

    sprintf(by_Buffer,"Y %d %d %d %d\r",xoffset,yoffset,xlen,ylen);

    write(i_AX25Connection,by_Buffer,strlen(by_Buffer));

    fprintf(stdout,"=> %s\n",by_Buffer);

    usleep(2000000);

    //  Set base counter for t_watchDog
    t_watchDog=time(NULL);

    while ((nread=read(i_AX25Connection,by_Buffer,MAXREAD))<=0)
    {
      if ((time(NULL)-t_watchDog)>WAIT_TIMEOUT)
      {
        // 30 seconds without rx anything!
          fprintf(stderr, "beginrxdata: timeout (%d seconds)\n",WAIT_TIMEOUT);
          goto discon; // disconnect node....
      }

      usleep(2000000);
    }

    if (nread>=4 && strcmp(by_Buffer,"*** "))
    {
      //check that no error reports occurred ("*** Napaci Parameters")
      // ok, it seems all is going well...
      for (x=0,y=0,nly=0;y<ylen;  )
      {
        //copy pixels to by_imageMap
        for (j=0;j<nread;j++)
        {
          by_imageMap[nly][x++]=by_Buffer[j];

          if (x>=xlen)
          {
            if (y && ((y%64)==0))
              fprintf(stderr,".\n");
            else
              fprintf(stderr,".");

            fflush(stderr);

            x=0;
            y++;
            nly++;

            if (y&1)
              nly++;
            else
            {
              //  add a row between y=0 and y=1
              //  lines: 0 interlace 1 - 2 interlace 3 - 4 interlace 5
              Deinterlace(by_imageMap, xlen, nly-3);
            }

            if (y>=ylen)
            {
              fprintf(stdout,"\nDownload completed\n");
              break;
            }
          }
        }

        if (y>=ylen)
          break;

        t_watchDog=time(NULL);

        while ((nread=read(i_AX25Connection,by_Buffer,MAXREAD))<=0)
        {
          if ((time(NULL)-t_watchDog)>WAIT_TIMEOUT)
          {
            // 40 seconds without rx anything!
            fprintf(stderr, "rxdata: timeout (%d seconds)\n",WAIT_TIMEOUT);
            goto discon; // disconnect node....
          }

          usleep(100000);
      }
    }

discon:
      // verify that i'm connected with the first node...
      usleep(1000000);
      read(i_AX25Connection,by_Buffer,MAXREAD);
      write(i_AX25Connection,"\r",1);
      sprintf(nodeprompt,"%s>",sz_SVNodeName);

      if (!AX25WaitString(i_AX25Connection, nodeprompt, sz_line, WAIT_TIMEOUT))
      {
        //error disconnecting node argv[i]
        fprintf(stderr, "Error disconnecting node %s: exit\n",sz_SVNodeName);
      }
    }
  }
  else
  {
    // Error connecting the first node! exit
    fprintf(stderr, "Error connecting the fist node %s\n",sz_SVNodeName);
  }

  // now disconnect to the first node!
  write(i_AX25Connection,"Q\r",2);

  fprintf(stdout,"=> Q\n");

  usleep(1000000);

  //  Close connection

  close(i_AX25Connection);

  //  Create output filename
  
  strcpy(sz_OutFile, sz_SVNodeName);
  strcat(sz_OutFile, ".jpeg");

  if(ConvertToJPEG(by_imageMap, xlen, nly, sz_OutFile, i_quality))
  {
    fprintf(stderr, "Error creating %s: exit\n",sz_OutFile);
    return  -1;
  }

  return EXIT_SUCCESS;
}


